unit ViewControllerU;

{$modeswitch ObjectiveC2}

interface

uses
  SysUtils, Foundation, AppKit, CoreLocation, Mapbox, NdfdLib;

type
  ViewController = objcclass(NSViewController, MGLMapViewDelegateProtocol,
                             NSGestureRecognizerDelegateProtocol)
     {IBOutlets}
    mapView : MGLMapView;

     {IBActions}

  public
    procedure viewDidLoad; override;
    procedure setRepresentedObject(representedObject: id); override;
    procedure dealloc; override;

    function mapView_annotationCanShowCallout(
              mapView: MGLMapView; annotation: MGLAnnotationProtocol): boolean;
              message 'mapView:annotationCanShowCallout:';
    procedure mapView_didDeselectAnnotation(
               mapView: MGLMapView; annotation: MGLAnnotationProtocol);
               message 'mapView:didDeselectAnnotation:';
    procedure handleClick(gesture: NSClickGestureRecognizer); message 'handleClick:';
  end;

var
  Ndfd : TNdfdLib;  {Global so it can be accessed by AppDelegate}


implementation

procedure ViewController.viewDidLoad;
var
  gesture : NSClickGestureRecognizer;
begin
  inherited viewDidLoad;

  mapView.setDelegate(self);

  mapView.compass.setHidden(True);

  Ndfd := TNdfdLib.Create;

  gesture := NSClickGestureRecognizer.alloc.initWithTarget_action(
              self, objcselector('handleClick:'));
  gesture.setDelegate(self);
  gesture.setNumberOfClicksRequired(2);
  mapView.addGestureRecognizer(gesture);
end;


procedure ViewController.setRepresentedObject(representedObject: id);
begin
 inherited setRepresentedObject(representedObject);
end;


procedure ViewController.dealloc;
begin
  Ndfd.Free;
  inherited dealloc;
end;


function ViewController.mapView_annotationCanShowCallout(
          mapView: MGLMapView; annotation: MGLAnnotationProtocol): boolean;
begin
  Result := True;
end;


procedure ViewController.mapView_didDeselectAnnotation(
           mapView: MGLMapView; annotation: MGLAnnotationProtocol);
begin
  mapView.removeAnnotation(annotation);
end;


procedure ViewController.handleClick(gesture: NSClickGestureRecognizer);
var
  screenLoc   : NSPoint;
  geoLoc      : CLLocationCoordinate2D;
  fmtSettings : TFormatSettings;
  marker      : MGLPointAnnotation;
begin
  screenLoc := gesture.locationInView(mapView);
  geoLoc := mapView.convertPoint_toCoordinateFromView(screenLoc, mapView);
  fmtSettings.DecimalSeparator := '.';
  if not Ndfd.LoadForecast(FloatToStr(geoLoc.latitude, fmtSettings),
                           FloatToStr(geoLoc.longitude, fmtSettings),
                           False) then
    WriteLn(Ndfd.GetLastError)
  else
    begin
    marker := MGLPointAnnotation.alloc.init;
    marker.setCoordinate(geoLoc);
    marker.setTitle(NSSTR('Today''s forecast'));
    marker.setSubtitle(NSSTR('High ' + Ndfd.GetMaxTemp(1) +
                             ', Low ' + Ndfd.GetMinTemp(1)));
    mapView.addAnnotation(marker);
    mapview.selectAnnotation(marker);
    end;
end;


end.
